@@ -74,14 +74,19 @@ class BokehDataFinder(BokehBaseExplorer):
         for _widget in self._search_watch_widgets():
             _widget.on_change(
                 "value",
-                lambda attr, old, new: self._trigger_selection_filters()
+                lambda attr, old, new: self._selection_stages_callback(
+                    "load", "write", "read"
+                )
                 if filter_flag()
                 else None,
             )
 
-        # active toggles always trigger selection filter
+        # change of toggles always trigger selection filter
         self.search_filter_box.on_change(
-            "active", lambda attr, old, new: self._trigger_selection_filters()
+            "active",
+            lambda attr, old, new: self._selection_stages_callback(
+                "load", "write", "read"
+            ),
         )
 
     def plot(self):
@@ -250,18 +255,22 @@ class BokehSoftLabelExplorer(BokehBaseExplorer):
         self.score_col = score_col
         super().__init__(df_dict, **kwargs)
 
-    def _build_tooltip(self, extra):
+    def _build_tooltip(self, specified):
         """
         ???+ note "On top of the parent method, add the soft label fields to the tooltip."
             | Param            | Type   | Description                  |
             | :--------------- | :----- | :--------------------------- |
-            | `extra`          | `str`  | user-supplied extra HTML |
+            | `specified`      | `str`  | user-specified HTML          |
+
+            Note that this is a method rather than a class attribute because
+            child classes may involve instance attributes in the tooltip.
         """
-        standard = bokeh_hover_tooltip(
-            **self.__class__.TOOLTIP_KWARGS,
-            custom={"Soft Label": self.label_col, "Soft Score": self.score_col},
-        )
-        return f"{standard}\n{extra}"
+        if not specified:
+            return bokeh_hover_tooltip(
+                **self.__class__.TOOLTIP_KWARGS,
+                custom={self.label_col: "Soft Label", self.score_col: "Soft Score"},
+            )
+        return specified
 
     def _mandatory_column_defaults(self):
         """
@@ -359,14 +368,19 @@ class BokehSoftLabelExplorer(BokehBaseExplorer):
         # when toggled as active, score range change triggers selection filter
         self.score_range.on_change(
             "value",
-            lambda attr, old, new: self._trigger_selection_filters()
+            lambda attr, old, new: self._selection_stages_callback(
+                "load", "write", "read"
+            )
             if filter_flag()
             else None,
         )
 
-        # active toggles always trigger selection filter
+        # changing toggles always re-evaluate selection filter
         self.score_filter_box.on_change(
-            "active", lambda attr, old, new: self._trigger_selection_filters()
+            "active",
+            lambda attr, old, new: self._selection_stages_callback(
+                "load", "write", "read"
+            ),
         )
 
     def plot(self, **kwargs):
@@ -686,6 +700,9 @@ class BokehSnorkelExplorer(BokehBaseExplorer):
                 ]
                 self.sources[_key].selected.indices = _kept
 
+            # selection reduced, need to trigger readall callbacks
+            self._selection_stages_callback("read")
+
         self.lf_filter_trigger.on_click(callback_filter)
 
     def _postprocess_sources(self):
@@ -741,7 +758,7 @@ class BokehSnorkelExplorer(BokehBaseExplorer):
         # remove from legend, checking that there is exactly one entry
         legend_idx_to_pop = None
         for i, _item in enumerate(self.figure.legend.items):
-            _label = _item.label.get("value", "")
+            _label = _item.label.value
             if _label == lf_name:
                 assert legend_idx_to_pop is None, f"Legend collision: {lf_name}"
                 legend_idx_to_pop = i
@@ -852,7 +869,7 @@ class BokehSnorkelExplorer(BokehBaseExplorer):
             view = self._view_correct(L_labeled)
             data_dict["glyphs"]["C"] = self.figure.square(
                 *xy_axes,
-                source=view.source,
+                source=self.sources["labeled"],
                 view=view,
                 name="labeled",
                 tags=[lf.name],
@@ -862,7 +879,7 @@ class BokehSnorkelExplorer(BokehBaseExplorer):
             view = self._view_incorrect(L_labeled)
             data_dict["glyphs"]["I"] = self.figure.x(
                 *xy_axes,
-                source=view.source,
+                source=self.sources["labeled"],
                 view=view,
                 name="labeled",
                 tags=[lf.name],
@@ -872,7 +889,7 @@ class BokehSnorkelExplorer(BokehBaseExplorer):
             view = self._view_missed(L_labeled, lf.targets)
             data_dict["glyphs"]["M"] = self.figure.cross(
                 *xy_axes,
-                source=view.source,
+                source=self.sources["labeled"],
                 view=view,
                 name="labeled",
                 tags=[lf.name],
@@ -882,7 +899,7 @@ class BokehSnorkelExplorer(BokehBaseExplorer):
             view = self._view_hit(L_raw)
             data_dict["glyphs"]["H"] = self.figure.circle(
                 *xy_axes,
-                source=view.source,
+                source=self.sources["raw"],
                 view=view,
                 name="raw",
                 tags=[lf.name],
